package kernel.ps;

import haxe.ds.ReadOnlyArray;
using tink.CoreApi;

typedef HandleConfig = {
	?args: Array<String>,
	?onWrite: Callback<String>,
	?onExit: Callback<Bool>,
}

class ProcessHandle {
	public var args(get,null): ReadOnlyArray<String>;

	private final config:HandleConfig;
	private final closeFuture: Future<Bool>;
	private var closeFutureResolev: Bool -> Void;

	@:allow(kernel.ps.ProcessManager)
	private function new(config: HandleConfig) {
		this.config = config;
		this.closeFuture = new Future<Bool>((trigger)->{
			this.closeFutureResolev = trigger;
			return null;
		});

		if (this.config.onExit != null) {
			this.closeFuture.handle(this.config.onExit);
		}
	}

	public function onExit(): Future<Bool> {
		return this.closeFuture;
	}

	public function close(success: Bool = true): Void {
		this.dispose();
		this.closeFutureResolev(success);
	}

	public function write(message: String): Void {
		this.config.onWrite.invoke(message);
	}

	public function writeLine(message: String): Void {
		this.write(message + "\n");
	}

	private function dispose() {
		// TODO
	}

	function get_args():ReadOnlyArray<String> {
		return this.config.args;
	}
}
